/* ... */
/* this file is part of netrafd project */
/* Copyright (c) 2005 M.S. */


#include "gui.h"
#include <panel.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>



  
  /**
    Thread "semaphore" allowing only one refresh-screen request
    to be done at one moment.
  */
  extern pthread_mutex_t refresh_locker;




/**
  Starts GUI (ncurses) mode and prepairing color_pairs.
*/
int begin_gui_mode(void){


  if(initscr() == NULL)
    return -1;
  if(start_color() == ERR)
    return -1;
  if(cbreak() == ERR)
    return -1;
  if(keypad(stdscr, TRUE) == ERR)
    return -1;
  if(noecho() == ERR)
    return -1;
  curs_set(0);

  init_pair(1, COLOR_YELLOW, COLOR_BLUE);
  init_pair(2, COLOR_BLACK, COLOR_CYAN);
  init_pair(3, COLOR_CYAN, COLOR_BLUE);
  init_pair(4, COLOR_WHITE, COLOR_BLUE);
  init_pair(5, COLOR_WHITE, COLOR_RED);
  init_pair(6, COLOR_BLACK, COLOR_BLUE);
  init_pair(7, COLOR_GREEN, COLOR_BLUE);
  init_pair(8, COLOR_WHITE, COLOR_CYAN);
  init_pair(9, COLOR_RED, COLOR_BLUE);
  init_pair(10, COLOR_GREEN, COLOR_BLACK);
  init_pair(11, COLOR_YELLOW, COLOR_BLACK);
  init_pair(12, COLOR_RED, COLOR_BLACK);

  ESCDELAY=0;

  return 0;
}




/**
  Ends GUI (ncurses) mode and flushing desktop.
*/
void end_gui_mode(void){


  draw_background(stdscr, ' ', COLOR_PAIR(0));
  update_screen();
  curs_set(1);
  endwin();

  return;
}




/**
  Used to refresh screen "thread-safely".
*/
__inline__ void update_screen(void){


  if(pthread_mutex_trylock(&refresh_locker) == 0){
    update_panels();
    doupdate();
    pthread_mutex_unlock(&refresh_locker);
  }

  return;
}




/**
  Fills window with one character.
  @param win  Pointer to WINDOW (curses) structure
  @param c  With char 'c' window 'win' will be filled
  @param attribute  ncurses COLOR_PAIR of given char
*/
void draw_background(WINDOW *win, char c, long attribute){
  int i, j, max_y, max_x;


  getmaxyx(win, max_y, max_x);
  wattron(win, attribute);
  for(i=0; i<max_y; i++)
    for(j=0; j<max_x; j++)
	mvwaddch(win, i, j, c);
  wattroff(win, attribute);

  return;
}




/**
  Prints horizontal bar with:
  @param text  Text to write
  @param y  Vertical position
  @param align  Horizontal text alignment (ALIGN_LEFT, ALIGN_CENTER, ALIGN_RIGHT)
  @param attribute  ncurses COLOR_PAIR
*/
void draw_bar(WINDOW *win, const char *text, int y, int align, long attribute){
  int i, max_y, max_x;
  int curr_y, curr_x;


  getyx(win, curr_y, curr_x);
  getmaxyx(win, max_y, max_x);
  wattron(win, attribute);
  for(i=0; i<max_x; i++)
    mvwaddch(win, y, i, ' ');
  if(align == ALIGN_LEFT){
    i = 0;
  }else{
    i = strlen(text);
    if(align == ALIGN_CENTER)
	i = (max_x/2) - (i/2);
    if(align == ALIGN_RIGHT)
	i = max_x - i;
  }
  mvwprintw(win, y, i, (char*)text);
  wattroff(win, attribute);
  wmove(win, curr_y, curr_x);

  return;
}




/**
  Used to delete all specified chars at end of string.
  @param text  Text to change
  @param ch  Char-pattern to delete from tail
*/
void delete_suffix(char *text, char ch){
  char *point;


  if((point=strrchr(text, ch)) == NULL)
    return;
  while(point-text == strlen(text)-1){
    *point = '\0';
    if((point=strrchr(text, ch)) == NULL)
      return;
  }    

  return;
}
