/* ... */
/* this file is part of netrafd project */
/* Copyright (c) 2005 M.S. */


#ifndef __NETRAFG_MENUS_
#define __NETRAFG_MENUS_ 1

#include <panel.h>
#include <menu.h>

#define NULL_TERMINATED_ARRAY_SIZE(out, a) for((out)=0; a[out]!=NULL; (out)++); (out)++




  /** menu structure */
  typedef struct MENU_T{
    /** pointer to menu structure */
    MENU *menu;

    /** array of pointers to menu items */
    ITEM **menu_items;

    /** window associated with menu structure */
    WINDOW *menu_win;

    /** panel structure associated with menu_win */
    PANEL *menu_panel;

    /** array of pointers to sub_menu structures */
    struct MENU_T **sub_menus;

    /** window representing status bar */
    WINDOW *status_win;

    /** panel structure associated with status_win */
    PANEL *status_panel;

  } MENU_T;




/**
  Looking through table of strings (NULL terminated)
  and returns length of longest string.
  @param strtable  Table of strings (NULL terminated)
  @return  length of longest string
*/
size_t longest_strlen(char **strtable);




/**
  Create 1-collumn menu.
  @param begin_y  Upper-left corner start-line
  @param begin_x  Upper-left corner start-collumn
  @param fore_attr  Foreground (selected item) ncurses color_pair
  @param back_attr  Background (everything besides selected item) ncurses color-pair
  @param choices  Pointer to string table of menu choices (NULL-terminated)
  @param descriptions  Pointer to string table of menu descriptions (NULL-terminated)
  @param visible_rows  If visible_rows<=0 then height of menu will be counted automatially,
                       otherwise this parameter describes maximum number of visible rows.
		       If number of menu items is greater than visible_rows parameter,
		       menu will be scrollable.
  @param forced_width  When forced_width>0, menu will have exact "forced_width" characters width
  @param winname  Frame around can be named with "winname" string. If you don't want
                  named frame pass NULL.
  @return  Pointer to allocated and initialized MENU_T structure or NULL if fail
*/
MENU_T *create_sub_menu(int begin_y,
		        int begin_x,
			chtype fore_attr,
			chtype back_attr,
			char **choices,
			char **descriptions,
			int visible_rows,
			int forced_width,
			char *winname);




/**
  Create main menu (bar) and all submenus.
  Beside first 4 arguments function takes as many arguments as main_choices table size.
  All of that arguments are in form:
    char **sub_menu_1_choices,  char **sub_menu_1_descriptions,
    char **sub_menu_2_choices,  char **sub_menu_2_descriptions,
                   ...
		   ...
    char **sub_menu_N_choices,  char **sub_menu_N_descriptions,
  @param fore_attr  Foreground (selected item) ncurses color_pair
  @param back_attr  Background (everything besides selected item) ncurses color-pair
  @param main_choices  Pointer to string table of main_menu choices (NULL-terminated)
  @param main_descriptions  Pointer to string table of main_menu descriptions (NULL-terminated)
  @return  Pointer to allocated and initialized MENU_T structure or NULL if fail
*/
MENU_T *create_menu(chtype fore_attr,
		    chtype back_attr,
		    char **main_choices,
		    char **main_descriptions,
		    ...);




/**
  Prints description of currently selected menu-item.
  @param menu  Pointer to MENU_T structure
               (it must be initialized with create_menu() function
*/
__inline__ void print_current_description(MENU_T *menu);




/**
  Command-processing loop for main menu system.
  For request and return values descriptions see menu_driver(3X).
  @param menu  Pointer to MENU_T structure
               (it must be initialized with create_menu() function
  @param request  Requests are as follows:
		  REQ_LEFT_ITEM, REQ_RIGHT_ITEM, REQ_UP_ITEM, REQ_DOWN_ITEM
  @return  E_OK if routine succeded
*/
int main_menu_driver(MENU_T *menu, int request);




/**
  Process user input on menu and returns user's choice.
  For return values descriptions see menu_driver(3X).
  @param menu  Pointer to MENU_T structure
               (it must be initialized with create_menu() function
  @param main  It is RETURN value - pointer to selected main ITEM structure
  @param main  It is RETURN value - pointer to selected sub ITEM structure
  @return  E_OK if routine succeded
*/
int process_main_menu(MENU_T *menu, ITEM **main, ITEM **sub);




/** 
  Frees all memory and clean data structures occupied by sub_menu.
  @param menu  Pointer to MENU_T structure
               (it must be initialized with create_sub_menu() function
*/
void destroy_sub_menu(MENU_T *menu);




/**
  Frees all memory and clean data structures occupied by menu.
  @param menu  Pointer to MENU_T structure
               (it must be initialized with create_menu() function
*/
void destroy_menu(MENU_T *menu);




#endif
